@@ -31,7 +31,7 @@
 ;; three passes.  Specifically, getting all of the cross references
 ;; made correctly is a lot of work.  Fortunately, texinfo-mode has
 ;; functions for building menus and updating nodes.  This saves a lot
-;; of sweat and is the principle reason why I decided to write this is
+;; of sweat and is the principle reason why I decided to write this in
 ;; emacs lisp.
 ;;
 ;; Everything else in gnuplot is written in C for the sake of
@@ -157,6 +157,7 @@ If PATTERN is omitted, it defaults to \"[ \\f\\t\\n\\r\\v]+\"."
 @defindex op
 @defindex tm
 
+@dircategory Math
 @direntry
 * GNUPLOT: (gnuplot).             An Interactive Plotting Program
 @end direntry
@@ -171,15 +172,15 @@ If PATTERN is omitted, it defaults to \"[ \\f\\t\\n\\r\\v]+\"."
 
             An Interactive Plotting Program
              Thomas Williams & Colin Kelley
-          Version 3.7 organized by: David Denholm
+          Version 4.2 organized by: Hans-Bernhard Broeker
 
- Copyright (C) 1986 - 1993, 1998   Thomas Williams, Colin Kelley
+ Copyright (C) 1986 - 1993, 1998, 2004   Thomas Williams, Colin Kelley
 
-       Mailing list for comments: info-gnuplot@@dartmouth.edu
-     Mailing list for bug reports: bug-gnuplot@@dartmouth.edu
+       Mailing list for comments: gnuplot-info@@lists.sourceforge.net
+     Mailing list for bug reports: gnuplot-bugs@@lists.sourceforge.net
 
-         This manual was prepared by Dick Crawford
-                   3 December 1998
+         This manual was originally prepared by Dick Crawford
+                   Version 4.2 - 1 Oct 2006
 
 
 Major contributors (alphabetic order):
@@ -243,15 +244,37 @@ Major contributors (alphabetic order):
 
 (defvar d2t-terminal-list ())
 (setq d2t-terminal-list
-      '("ai"
+      '("aed"
+	"ai"
+	"amiga"
+	"apollo"
+	"aquaterm"
+	"atariaes"
+	"atarivdi"
+	"be"
+	"cgi"
 	"cgm"
 	"corel"
+	"debug"
+	"djsvga"
 	"dumb"
 	"dxf"
+	"dxy"
 	"eepic"
+	"emf"
+	"emxvga"
+	"epslatex"
 	"epson"
+	"excl"
+	"fg"
 	"fig"
+	"gd"
+	"ggi"
 	"gif"
+	"gnugraph"
+	"gpic"
+	"gpr"
+	"grass"
 	"hp26"
 	"hp2648"
 	"hp500c"
@@ -259,20 +282,46 @@ Major contributors (alphabetic order):
 	"hpljii"
 	"hppj"
 	"imagen"
+	"iris4d"
+	"jpeg"
+	"kyo"
 	"latex"
+	"linux"
+	"mac"
 	"metafont"
+	"metapost"
+	"mgr"
 	"mif"
+	"multitos"
+	"next"
+	"openstep"
 	"pbm"
+	"pc"
+	"pdf"
 	"png"
 	"post"
 	"pslatex"
 	"pstricks"
 	"qms"
+	"regis"
+	"rgip"
+	"sun"
+	"svg"
+	"t410x"
 	"table"
 	"tek"
 	"texdraw"
+	"tgif"
 	"tkcanvas"
-	"tpic"))
+	"tpic"
+	"unixpc"
+	"unixplot"
+	"v384"
+	"vgagl"
+	"vws"
+	"win"
+	"x11"
+	"xlib"))
 
 (defun d2t-doc-to-texi-verbosely ()
   "Run `d2t-doc-to-texi' noisily"
@@ -324,9 +373,9 @@ particular conversion chore."
     (search-forward "@node")
     ;; (beginning-of-line)
     ;; (insert "\n\n" d2t-main-menu "\n\n")
-    (search-forward "@node Old_bugs")	; `texinfo-all-menus-update' seems
+    (search-forward "@node Bugs")	; `texinfo-all-menus-update' seems
     (beginning-of-line)			; to miss this one.  how odd.
-    (insert "@menu\n* Old_bugs::\t\t\t\n@end menu\n\n")
+    (insert "@menu\n* Bugs::\t\t\t\n@end menu\n\n")
     (goto-char (point-max))
     (insert d2t-texi-footer))
   (load-library "texinfo") ;; now do the hard stuff with texinfo-mode
@@ -368,7 +417,7 @@ particular conversion chore."
 					'("atarivdi" "multitos" "atariaes"))))
     (if (string-match "mac" system-configuration)
 	(setq d2t-terminal-list (append d2t-terminal-list
-					'("mac"))))
+					'("mac" "openstep"))))
     (if (string-match "beos" system-configuration)
 	(setq d2t-terminal-list (append d2t-terminal-list
 					'("be"))))
@@ -404,7 +453,7 @@ particular conversion chore."
   (setq d2t-terminal-list (sort d2t-terminal-list 'string<))
   (let ((list d2t-terminal-list) file node marker)
     (save-excursion
-      (when (re-search-forward "^<4" (point-max) t)
+      (when (re-search-forward "^<3" (point-max) t)
 	(beginning-of-line)
 	(insert "@c ")
 	(forward-line 1)
@@ -587,9 +636,10 @@ comment out the multi-word ? entries."
   (and d2t-verbose (message "  Doing d2t-indexing ..."))
   (save-excursion
     (while (not (eobp))
-      (re-search-forward "^\\\?\\([^ \n]+\\) *$" (point-max) "to_end")
+      (re-search-forward "^\\(\\\?\\([^ \n]+\\)\\|=\\([^\n]+\\)\\) *$" 
+			 (point-max) "to_end")
       (unless (eobp)
-	(let ((word (match-string 1))
+	(let ((word (or (match-string 2) (match-string 3)))
 	      (eol  (save-excursion (end-of-line) (point-marker))))
 	  (beginning-of-line)
 	  (delete-region (point-marker) eol)
@@ -720,7 +770,8 @@ This must be run after `d2t-first-column'."
 					 ","
 					 (remove* ?^ (match-string 2)
 						  :test 'char-equal)
-					 "}"))))
+					 "}"))
+		  (insert "@c ")))
 	     ;; translate <ul> </ul> to @itemize environment
 	     ((and (string= bracket "<") (string-match "^ul" tag))
 	      (delete-region (point) eol)
