@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: tables.c,v 1.50 2004/04/13 17:24:01 broeker Exp $"); }
+static char *RCSid() { return RCSid("$Id: tables.c,v 1.71 2006/06/15 15:42:34 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - tables.c */
@@ -41,16 +41,15 @@ static char *RCSid() { return RCSid("$Id: tables.c,v 1.50 2004/04/13 17:24:01 br
 #include "setshow.h"
 #include "term_api.h"
 #include "util.h"
-
-#ifdef PM3D
 # include "getcolor.h"
-#endif
 
 /* gnuplot commands */
 
 /* the actual commands */
 const struct gen_ftable command_ftbl[] =
 {
+    { "ra$ise", raise_command },
+    { "low$er", lower_command },
 #ifdef USE_MOUSE
     { "bi$nd", bind_command },
 #endif
@@ -154,11 +153,11 @@ const struct gen_table set_tbl[] =
     { "ac$tion_table", S_ACTIONTABLE },
     { "at", S_ACTIONTABLE },
     { "an$gles", S_ANGLES },
-    { "ar$row", S_ARROW }, 
-    { "au$toscale", S_AUTOSCALE }, 
+    { "ar$row", S_ARROW },
+    { "au$toscale", S_AUTOSCALE },
     { "b$ars", S_BARS },
     { "bor$der", S_BORDER },
-    { "box$width", S_BOXWIDTH }, 
+    { "box$width", S_BOXWIDTH },
     { "cl$abel", S_CLABEL },
     { "c$lip", S_CLIP },
     { "cn$trparam", S_CNTRPARAM },
@@ -166,7 +165,7 @@ const struct gen_table set_tbl[] =
     { "da$ta", S_DATA },
 
     { "data$file", S_DATAFILE },
-    
+
     { "dg$rid3d", S_DGRID3D },
     { "du$mmy", S_DUMMY },
     { "enc$oding", S_ENCODING },
@@ -178,9 +177,7 @@ const struct gen_table set_tbl[] =
     { "fu$nctions", S_FUNCTIONS },
     { "g$rid", S_GRID },
     { "hid$den3d", S_HIDDEN3D },
-#if defined(HAVE_LIBREADLINE) && defined(GNUPLOT_HISTORY)
     { "his$torysize", S_HISTORYSIZE },
-#endif
     { "is$osamples", S_ISOSAMPLES },
     { "k$ey", S_KEY },
     { "keyt$itle", S_KEYTITLE },
@@ -190,6 +187,9 @@ const struct gen_table set_tbl[] =
     { "loa$dpath", S_LOADPATH },
     { "loc$ale", S_LOCALE },
     { "log$scale", S_LOGSCALE },
+#ifdef GP_MACROS
+    { "mac$ros", S_MACROS },
+#endif
     { "map$ping", S_MAPPING },
     { "map$ping3d", S_MAPPING },
 
@@ -214,28 +214,25 @@ const struct gen_table set_tbl[] =
     { "nomy2t$ics", S_NOMY2TICS },
     { "mzt$ics", S_MZTICS },
     { "nomzt$ics", S_NOMZTICS },
-#ifdef PM3D
     { "mcbt$ics", S_MCBTICS },
     { "nomcbt$ics", S_NOMCBTICS },
-#endif
-
     { "of$fsets", S_OFFSETS },
     { "or$igin", S_ORIGIN },
     { "o$utput", SET_OUTPUT },
     { "pa$rametric", S_PARAMETRIC },
-#ifdef PM3D
     { "pm$3d", S_PM3D },
     { "pal$ette", S_PALETTE },
     { "colorb$ox", S_COLORBOX },
-#endif
     { "p$lot", S_PLOT },
     { "poi$ntsize", S_POINTSIZE },
     { "pol$ar", S_POLAR },
     { "pr$int", S_PRINT },
+    { "obj$ect", S_OBJECT },
     { "sa$mples", S_SAMPLES },
     { "si$ze", S_SIZE },
     { "st$yle", S_STYLE },
     { "su$rface", S_SURFACE },
+    { "table", S_TABLE },
     { "t$erminal", S_TERMINAL },
     { "termo$ptions", S_TERMOPTIONS },
     { "ti$cs", S_TICS },
@@ -247,24 +244,21 @@ const struct gen_table set_tbl[] =
     { "v$ariables", S_VARIABLES },
     { "ve$rsion", S_VERSION },
     { "vi$ew", S_VIEW },
+    { "xyp$lane", S_TICSLEVEL },
 
     { "xda$ta", S_XDATA },
     { "x2da$ta", S_X2DATA },
     { "yda$ta", S_YDATA },
     { "y2da$ta", S_Y2DATA },
     { "zda$ta", S_ZDATA },
-#ifdef PM3D
     { "cbda$ta", S_CBDATA },
-#endif
 
     { "xl$abel", S_XLABEL },
     { "x2l$abel", S_X2LABEL },
     { "yl$abel", S_YLABEL },
     { "y2l$abel", S_Y2LABEL },
     { "zl$abel", S_ZLABEL },
-#ifdef PM3D
     { "cbl$abel", S_CBLABEL },
-#endif
 
     { "xti$cs", S_XTICS },
     { "noxti$cs", S_NOXTICS },
@@ -276,10 +270,8 @@ const struct gen_table set_tbl[] =
     { "noy2ti$cs", S_NOY2TICS },
     { "zti$cs", S_ZTICS },
     { "nozti$cs", S_NOZTICS },
-#ifdef PM3D
     { "cbti$cs", S_CBTICS },
     { "nocbti$cs", S_NOCBTICS },
-#endif
 
     { "xdti$cs", S_XDTICS },
     { "noxdti$cs", S_NOXDTICS },
@@ -291,10 +283,8 @@ const struct gen_table set_tbl[] =
     { "noy2dti$cs", S_NOY2DTICS },
     { "zdti$cs", S_ZDTICS },
     { "nozdti$cs", S_NOZDTICS },
-#ifdef PM3D
     { "cbdti$cs", S_CBDTICS },
     { "nocbdti$cs", S_NOCBDTICS },
-#endif
 
     { "xmti$cs", S_XMTICS },
     { "noxmti$cs", S_NOXMTICS },
@@ -306,19 +296,15 @@ const struct gen_table set_tbl[] =
     { "noy2mti$cs", S_NOY2MTICS },
     { "zmti$cs", S_ZMTICS },
     { "nozmti$cs", S_NOZMTICS },
-#ifdef PM3D
     { "cbmti$cs", S_CBMTICS },
     { "nocbmti$cs", S_NOCBMTICS },
-#endif
 
     { "xr$ange", S_XRANGE },
     { "x2r$ange", S_X2RANGE },
     { "yr$ange", S_YRANGE },
     { "y2r$ange", S_Y2RANGE },
     { "zr$ange", S_ZRANGE },
-#ifdef PM3D
     { "cbr$ange", S_CBRANGE },
-#endif
     { "rr$ange", S_RRANGE },
     { "tr$ange", S_TRANGE },
     { "ur$ange", S_URANGE },
@@ -328,6 +314,7 @@ const struct gen_table set_tbl[] =
     { "x2zeroa$xis", S_X2ZEROAXIS },
     { "yzeroa$xis", S_YZEROAXIS },
     { "y2zeroa$xis", S_Y2ZEROAXIS },
+    { "zzeroa$xis", S_ZZEROAXIS },
     { "zeroa$xis", S_ZEROAXIS },
 
     { "z$ero", S_ZERO },
@@ -360,13 +347,26 @@ const struct gen_table set_key_tbl[] =
     { "b$ottom", S_KEY_BOTTOM },
     { "l$eft", S_KEY_LEFT },
     { "r$ight", S_KEY_RIGHT },
+    { "c$enter", S_KEY_CENTER },
+    { "ver$tical", S_KEY_VERTICAL },
+    { "hor$izontal", S_KEY_HORIZONTAL },
+    { "ov$er", S_KEY_OVER },
+    { "ab$ove", S_KEY_ABOVE },
     { "u$nder", S_KEY_UNDER },
-    { "be$low", S_KEY_UNDER },
+    { "be$low", S_KEY_BELOW },
+    { "at", S_KEY_MANUAL },
+    { "ins$ide", S_KEY_INSIDE },
     { "o$utside", S_KEY_OUTSIDE },
+    { "tm$argin", S_KEY_TMARGIN },
+    { "bm$argin", S_KEY_BMARGIN },
+    { "lm$argin", S_KEY_LMARGIN },
+    { "rm$argin", S_KEY_RMARGIN },
     { "L$eft", S_KEY_LLEFT },
     { "R$ight", S_KEY_RRIGHT },
     { "rev$erse", S_KEY_REVERSE },
     { "norev$erse", S_KEY_NOREVERSE },
+    { "inv$ert", S_KEY_INVERT },
+    { "noinv$ert", S_KEY_NOINVERT },
     { "enh$anced", S_KEY_ENHANCED },
     { "noenh$anced", S_KEY_NOENHANCED },
     { "b$ox", S_KEY_BOX },
@@ -390,7 +390,6 @@ const struct gen_table test_tbl[] =
     { NULL, TEST_INVALID }
 };
 
-#ifdef PM3D
 /* 'set colorbox' options */
 const struct gen_table set_colorbox_tbl[] =
 {
@@ -412,6 +411,7 @@ const struct gen_table set_palette_tbl[] =
     { "pos$itive",	S_PALETTE_POSITIVE },
     { "neg$ative",	S_PALETTE_NEGATIVE },
     { "gray",		S_PALETTE_GRAY },
+    { "grey",		S_PALETTE_GRAY },
     { "col$or",		S_PALETTE_COLOR },
     { "rgb$formulae",	S_PALETTE_RGBFORMULAE },
     { "def$ined",       S_PALETTE_DEFINED },
@@ -426,13 +426,13 @@ const struct gen_table set_palette_tbl[] =
 };
 
 
-const struct gen_table color_model_tbl[] = 
+const struct gen_table color_model_tbl[] =
 {
     { "RGB", C_MODEL_RGB },
-    { "HSV", C_MODEL_HSV }, 
-    { "CMY", C_MODEL_CMY }, 
-    { "YIQ", C_MODEL_YIQ }, 
-    { "XYZ", C_MODEL_XYZ }, 
+    { "HSV", C_MODEL_HSV },
+    { "CMY", C_MODEL_CMY },
+    { "YIQ", C_MODEL_YIQ },
+    { "XYZ", C_MODEL_XYZ },
     { NULL,  -1 }
 };
 
@@ -440,9 +440,11 @@ const struct gen_table color_model_tbl[] =
 const struct gen_table set_pm3d_tbl[] =
 {
     { "at",		S_PM3D_AT },
+    { "interp$olate",	S_PM3D_INTERPOLATE },
     { "scansfor$ward",	S_PM3D_SCANSFORWARD },
     { "scansback$ward", S_PM3D_SCANSBACKWARD },
     { "scansauto$matic",S_PM3D_SCANS_AUTOMATIC },
+    { "dep$thorder",    S_PM3D_DEPTH },
     { "fl$ush",		S_PM3D_FLUSH },
     { "ftr$iangles",	S_PM3D_FTRIANGLES },
     { "noftr$iangles",	S_PM3D_NOFTRIANGLES },
@@ -466,22 +468,36 @@ const struct gen_table set_pm3d_tbl[] =
 /* fixed RGB color names for 'set palette defined' */
 const struct gen_table pm3d_color_names_tbl[] =
 {
-    /* black, white and gray */
+    /* black, white and gray/grey */
     { "white"           , 255*(1<<16) + 255*(1<<8) + 255 },
     { "black"           ,   0*(1<<16) +   0*(1<<8) +   0 },
     { "gray0"           ,   0*(1<<16) +   0*(1<<8) +   0 },
+    { "grey0"           ,   0*(1<<16) +   0*(1<<8) +   0 },
     { "gray10"          ,  26*(1<<16) +  26*(1<<8) +  26 },
+    { "grey10"          ,  26*(1<<16) +  26*(1<<8) +  26 },
+    { "gray20"          ,  51*(1<<16) +  51*(1<<8) +  51 },
     { "grey20"          ,  51*(1<<16) +  51*(1<<8) +  51 },
+    { "gray30"          ,  77*(1<<16) +  77*(1<<8) +  77 },
     { "grey30"          ,  77*(1<<16) +  77*(1<<8) +  77 },
     { "gray40"          , 102*(1<<16) + 102*(1<<8) + 102 },
+    { "grey40"          , 102*(1<<16) + 102*(1<<8) + 102 },
     { "gray50"          , 127*(1<<16) + 127*(1<<8) + 127 },
+    { "grey50"          , 127*(1<<16) + 127*(1<<8) + 127 },
     { "gray60"          , 153*(1<<16) + 153*(1<<8) + 153 },
+    { "grey60"          , 153*(1<<16) + 153*(1<<8) + 153 },
+    { "gray70"          , 179*(1<<16) + 179*(1<<8) + 179 },
     { "grey70"          , 179*(1<<16) + 179*(1<<8) + 179 },
+    { "gray80"          , 204*(1<<16) + 204*(1<<8) + 204 },
     { "grey80"          , 204*(1<<16) + 204*(1<<8) + 204 },
     { "gray90"          , 229*(1<<16) + 229*(1<<8) + 229 },
+    { "grey90"          , 229*(1<<16) + 229*(1<<8) + 229 },
+    { "gray100"         , 255*(1<<16) + 255*(1<<8) + 255 },
     { "grey100"         , 255*(1<<16) + 255*(1<<8) + 255 },
+    { "gray"            , 190*(1<<16) + 190*(1<<8) + 190 },
     { "grey"            , 190*(1<<16) + 190*(1<<8) + 190 },
+    { "light-gray"      , 211*(1<<16) + 211*(1<<8) + 211 },
     { "light-grey"      , 211*(1<<16) + 211*(1<<8) + 211 },
+    { "dark-gray"       , 169*(1<<16) + 169*(1<<8) + 169 },
     { "dark-grey"       , 169*(1<<16) + 169*(1<<8) + 169 },
     /* red */
     { "red"             , 255*(1<<16) +   0*(1<<8) +   0 },
@@ -535,10 +551,10 @@ const struct gen_table pm3d_color_names_tbl[] =
     { "aquamarine"      , 127*(1<<16) + 255*(1<<8) + 212 },
     { "khaki"           , 240*(1<<16) + 230*(1<<8) + 140 },
     { "dark-khaki"      , 189*(1<<16) + 183*(1<<8) + 107 },
-    { "gold"            , 255*(1<<16) + 215*(1<<8) +   0 },
     { "goldenrod"       , 218*(1<<16) + 165*(1<<8) +  32 },
     { "light-goldenrod" , 238*(1<<16) + 221*(1<<8) + 130 },
     { "dark-goldenrod"  , 184*(1<<16) + 134*(1<<8) +  11 },
+    { "gold"            , 255*(1<<16) + 215*(1<<8) +   0 },
     { "beige"           , 245*(1<<16) + 245*(1<<8) + 220 },
     { "brown"           , 165*(1<<16) +  42*(1<<8) +  42 },
     { "orange"          , 255*(1<<16) + 165*(1<<8) +   0 },
@@ -550,8 +566,6 @@ const struct gen_table pm3d_color_names_tbl[] =
     { NULL, -1 }
 };
 
-#endif
-
 const struct gen_table show_style_tbl[] =
 {
     { "d$ata", SHOW_STYLE_DATA },
@@ -560,6 +574,11 @@ const struct gen_table show_style_tbl[] =
     { "fill", SHOW_STYLE_FILLING },
     { "fs", SHOW_STYLE_FILLING },
     { "ar$row", SHOW_STYLE_ARROW },
+    { "incr$ement", SHOW_STYLE_INCREMENT },
+#ifdef EAM_HISTOGRAMS
+    { "hist$ogram", SHOW_STYLE_HISTOGRAM },
+#endif
+    { "rect$angle", SHOW_STYLE_RECTANGLE },
     { NULL, SHOW_STYLE_INVALID }
 };
 
@@ -580,9 +599,10 @@ const struct gen_table plotstyle_tbl[] =
     { "xe$rrorbars", XERRORBARS },
     { "xye$rrorbars", XYERRORBARS },
     { "boxes", BOXES },
-#ifdef PM3D
-    { "filledc$urves", FILLEDCURVES },
+#ifdef EAM_HISTOGRAMS
+    { "hist$ograms", HISTOGRAMS },
 #endif
+    { "filledc$urves", FILLEDCURVES },
     { "boxer$rorbars", BOXERROR },
     { "boxx$yerrorbars", BOXXYERROR },
     { "st$eps", STEPS },
@@ -591,13 +611,17 @@ const struct gen_table plotstyle_tbl[] =
     { "vec$tors", VECTOR },
     { "fin$ancebars", FINANCEBARS },
     { "can$dlesticks", CANDLESTICKS },
-#ifdef PM3D
     { "pm$3d", PM3DSURFACE },
+#ifdef EAM_DATASTRINGS
+    { "labels", LABELPOINTS },
+#endif
+#ifdef WITH_IMAGE
+    { "ima$ge", IMAGE },
+    { "rgbima$ge", RGBIMAGE },
 #endif
     { NULL, -1 }
 };
 
-#ifdef PM3D
 const struct gen_table filledcurves_opts_tbl[] =
 {
     { "c$losed", FILLEDCURVES_CLOSED },
@@ -606,14 +630,13 @@ const struct gen_table filledcurves_opts_tbl[] =
     { "x2", FILLEDCURVES_X2 },
     { "y2", FILLEDCURVES_Y2 },
     { "xy", FILLEDCURVES_ATXY },
+    { "above", FILLEDCURVES_ABOVE },
+    { "below", FILLEDCURVES_BELOW },
     { NULL, -1 }
 };
-#endif
 
 int
-lookup_table(tbl, find_token)
-    const struct gen_table *tbl;
-    int find_token;
+lookup_table(const struct gen_table *tbl, int find_token)
 {
     while (tbl->key) {
 	if (almost_equals(find_token, tbl->key))
@@ -624,9 +647,7 @@ lookup_table(tbl, find_token)
 }
 
 parsefuncp_t
-lookup_ftable(ftbl, find_token)
-    const struct gen_ftable *ftbl;
-    int find_token;
+lookup_ftable(const struct gen_ftable *ftbl, int find_token)
 {
     while (ftbl->key) {
 	if (almost_equals(find_token, ftbl->key))
@@ -636,14 +657,12 @@ lookup_ftable(ftbl, find_token)
     return ftbl->value;
 }
 
-/* Returns index of the table tbl whose key matches the beginning of the 
+/* Returns index of the table tbl whose key matches the beginning of the
  * search string search_str.
  * It returns index into the table or -1 if there is no match.
- */ 
+ */
 int
-lookup_table_nth(tbl, search_str)
-    const struct gen_table *tbl;
-    const char *search_str;
+lookup_table_nth(const struct gen_table *tbl, const char *search_str)
 {
     int k = -1;
     while (tbl[++k].key)
@@ -652,17 +671,17 @@ lookup_table_nth(tbl, search_str)
     return -1; /* not found */
 }
 
-/* Returns index of the table tbl whose key matches the beginning of the 
+/* Returns index of the table tbl whose key matches the beginning of the
  * search string search_str. The table_len is necessary because the table
  * is searched in the reversed order. The routine is used in parsing commands
  * '(un)set log x2zcb', for instance.
  * It returns index into the table or -1 if there is no match.
- */ 
+ */
 int
-lookup_table_nth_reverse(tbl, table_len, search_str)
-    const struct gen_table *tbl;
-    int table_len;
-    const char *search_str;
+lookup_table_nth_reverse(
+    const struct gen_table *tbl,
+    int table_len,
+    const char *search_str)
 {
     while (--table_len >= 0) {
 	if (tbl[table_len].key && !strncmp(search_str, tbl[table_len].key, strlen(tbl[table_len].key)))
@@ -671,3 +690,15 @@ lookup_table_nth_reverse(tbl, table_len, search_str)
     return -1; /* not found */
 }
 
+/* Returns the key associated with this indexed value
+ * or NULL if the key/value pair is not found.
+ */
+const char *
+reverse_table_lookup(const struct gen_table *tbl, int entry)
+{
+    int k = -1;
+    while (tbl[++k].key)
+	if (tbl[k].value == entry)
+	    return(tbl[k].key);
+    return NULL;
+}
