@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: unset.c,v 1.99 2006/06/29 19:36:44 mikulik Exp $"); }
+static char *RCSid() { return RCSid("$Id: unset.c,v 1.99.2.2 2007/06/07 16:27:38 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - unset.c */
@@ -927,7 +927,7 @@ unset_label()
 		return;		/* exit, our job is done */
 	    }
 	}
-	int_error(c_token, "label not found");
+	/* int_warn(c_token, "label not found"); */
     }
 }
 
@@ -1331,9 +1331,22 @@ unset_style()
 	c_token++;
 	break;
     case SHOW_STYLE_LINE:
-	while (first_linestyle != NULL)
-	    delete_linestyle((struct linestyle_def *) NULL, first_linestyle);
 	c_token++;
+	if (END_OF_COMMAND) {
+	    while (first_linestyle != NULL)
+		delete_linestyle((struct linestyle_def *) NULL, first_linestyle);
+	} else {
+	    struct value a;
+	    int tag = (int) real(const_express(&a));
+	    struct linestyle_def *this, *prev;
+	    for (this = first_linestyle, prev = NULL; this != NULL; 
+		 prev = this, this = this->next) {
+		if (this->tag == tag) {
+		    delete_linestyle(prev, this);
+		    break;
+		}
+	    }
+	}
 	break;
     case SHOW_STYLE_FILLING:
 	unset_fillstyle();
