@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: util.c,v 1.38 2004/04/13 17:24:02 broeker Exp $"); }
+static char *RCSid() { return RCSid("$Id: util.c,v 1.65.2.2 2007/04/02 04:06:27 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - util.c */
@@ -38,7 +38,7 @@ static char *RCSid() { return RCSid("$Id: util.c,v 1.38 2004/04/13 17:24:02 broe
 
 #include "alloc.h"
 #include "command.h"
-#include "datafile.h"		/* for df_showdata */
+#include "datafile.h"		/* for df_showdata and df_reset_after_error */
 #include "misc.h"
 #include "plot.h"
 /*  #include "setshow.h" */		/* for month names etc */
@@ -51,27 +51,37 @@ static char *RCSid() { return RCSid("$Id: util.c,v 1.38 2004/04/13 17:24:02 broe
 # include <windows.h>
 #endif
 
+#if defined(HAVE_PWD_H)
+# include <sys/types.h>
+# include <pwd.h>
+#elif defined(_Windows)
+# include <windows.h>
+# if !defined(INFO_BUFFER_SIZE)
+#  define INFO_BUFFER_SIZE 32767
+# endif
+#endif
+
 /* Exported (set-table) variables */
 
 /* decimal sign */
 char *decimalsign = NULL;
 
+const char *current_prompt = NULL; /* to be set by read_line() */
 
 /* internal prototypes */
 
-static void parse_esc __PROTO((char *instr));
 static void mant_exp __PROTO((double, double, TBOOLEAN, double *, int *, const char *));
+static void parse_sq __PROTO((char *));
 
+#if 0 /* UNUSED */
 /*
  * chr_in_str() compares the characters in the string of token number t_num
  * with c, and returns TRUE if a match was found.
  */
 int
-chr_in_str(t_num, c)
-int t_num;
-int c;
+chr_in_str(int t_num, int c)
 {
-    register int i;
+    int i;
 
     if (!token[t_num].is_token)
 	return (FALSE);		/* must be a value--can't be equal */
@@ -81,23 +91,21 @@ int c;
     }
     return FALSE;
 }
-
+#endif
 
 /*
  * equals() compares string value of token number t_num with str[], and
  *   returns TRUE if they are identical.
  */
 int
-equals(t_num, str)
-int t_num;
-const char *str;
+equals(int t_num, const char *str)
 {
-    register int i;
+    int i;
 
     if (!token[t_num].is_token)
 	return (FALSE);		/* must be a value--can't be equal */
     for (i = 0; i < token[t_num].length; i++) {
-	if (input_line[token[t_num].start_index + i] != str[i])
+	if (gp_input_line[token[t_num].start_index + i] != str[i])
 	    return (FALSE);
     }
     /* now return TRUE if at end of str[], FALSE if not */
@@ -111,21 +119,19 @@ const char *str;
  *   returns TRUE if they are identical up to the first $ in str[].
  */
 int
-almost_equals(t_num, str)
-int t_num;
-const char *str;
+almost_equals(int t_num, const char *str)
 {
-    register int i;
-    register int after = 0;
-    register int start = token[t_num].start_index;
-    register int length = token[t_num].length;
+    int i;
+    int after = 0;
+    int start = token[t_num].start_index;
+    int length = token[t_num].length;
 
     if (!str)
 	return FALSE;
     if (!token[t_num].is_token)
 	return FALSE;		/* must be a value--can't be equal */
     for (i = 0; i < length + after; i++) {
-	if (str[i] != input_line[start + i]) {
+	if (str[i] != gp_input_line[start + i]) {
 	    if (str[i] != '$')
 		return (FALSE);
 	    else {
@@ -143,31 +149,40 @@ const char *str;
 
 
 int
-isstring(t_num)
-int t_num;
+isstring(int t_num)
 {
 
     return (token[t_num].is_token &&
-	    (input_line[token[t_num].start_index] == '\'' ||
-	     input_line[token[t_num].start_index] == '"'));
+	    (gp_input_line[token[t_num].start_index] == '\'' ||
+	     gp_input_line[token[t_num].start_index] == '"'));
 }
 
+int
+type_udv(int t_num)
+{
+    struct udvt_entry **udv_ptr = &first_udv;
+
+    while (*udv_ptr) {
+       if (equals(t_num, (*udv_ptr)->udv_name))
+	   return (*udv_ptr)->udv_value.type;
+       udv_ptr = &((*udv_ptr)->next_udv);
+    }
+    return 0;
+}
 
 int
-isanumber(t_num)
-int t_num;
+isanumber(int t_num)
 {
     return (!token[t_num].is_token);
 }
 
 
 int
-isletter(t_num)
-int t_num;
+isletter(int t_num)
 {
     return (token[t_num].is_token &&
-	    ((isalpha((unsigned char) input_line[token[t_num].start_index])) ||
-	     (input_line[token[t_num].start_index] == '_')));
+	    ((isalpha((unsigned char) gp_input_line[token[t_num].start_index])) ||
+	     (gp_input_line[token[t_num].start_index] == '_')));
 }
 
 
@@ -178,8 +193,7 @@ int t_num;
  *   identifier ( identifer {,identifier} ) =
  */
 int
-is_definition(t_num)
-int t_num;
+is_definition(int t_num)
 {
     /* variable? */
     if (isletter(t_num) && equals(t_num + 1, "="))
@@ -207,14 +221,11 @@ int t_num;
  *   a null.  No more than max chars are copied (including \0).
  */
 void
-copy_str(str, t_num, max)
-char *str;
-int t_num;
-int max;
+copy_str(char *str, int t_num, int max)
 {
-    register int i = 0;
-    register int start = token[t_num].start_index;
-    register int count = token[t_num].length;
+    int i = 0;
+    int start = token[t_num].start_index;
+    int count = token[t_num].length;
 
     if (count >= max) {
 	count = max - 1;
@@ -222,7 +233,7 @@ int max;
     }
 
     do {
-	str[i++] = input_line[start++];
+	str[i++] = gp_input_line[start++];
     } while (i != count);
     str[i] = NUL;
 
@@ -230,8 +241,7 @@ int max;
 
 /* length of token string */
 size_t
-token_len(t_num)
-int t_num;
+token_len(int t_num)
 {
     return (size_t)(token[t_num].length);
 }
@@ -242,14 +252,11 @@ int t_num;
  *   efficency.
  */
 void
-quote_str(str, t_num, max)
-char *str;
-int t_num;
-int max;
+quote_str(char *str, int t_num, int max)
 {
-    register int i = 0;
-    register int start = token[t_num].start_index + 1;
-    register int count;
+    int i = 0;
+    int start = token[t_num].start_index + 1;
+    int count;
 
     if ((count = token[t_num].length - 2) >= max) {
 	count = max - 1;
@@ -257,35 +264,34 @@ int max;
     }
     if (count > 0) {
 	do {
-	    str[i++] = input_line[start++];
+	    str[i++] = gp_input_line[start++];
 	} while (i != count);
     }
     str[i] = NUL;
     /* convert \t and \nnn (octal) to char if in double quotes */
-    if (input_line[token[t_num].start_index] == '"')
+    if (gp_input_line[token[t_num].start_index] == '"')
 	parse_esc(str);
+    else
+        parse_sq(str);
 }
 
 
 /*
- * capture() copies into str[] the part of input_line[] which lies between
+ * capture() copies into str[] the part of gp_input_line[] which lies between
  * the begining of token[start] and end of token[end].
  */
 void
-capture(str, start, end, max)
-char *str;
-int start, end;
-int max;
+capture(char *str, int start, int end, int max)
 {
-    register int i, e;
+    int i, e;
 
     e = token[end].start_index + token[end].length;
     if (e - token[start].start_index >= max) {
 	e = token[start].start_index + max - 1;
 	FPRINTF((stderr, "str buffer overflow in capture"));
     }
-    for (i = token[start].start_index; i < e && input_line[i] != NUL; i++)
-	*str++ = input_line[i];
+    for (i = token[start].start_index; i < e && gp_input_line[i] != NUL; i++)
+	*str++ = gp_input_line[i];
     *str = NUL;
 }
 
@@ -295,18 +301,16 @@ int max;
  * string.
  */
 void
-m_capture(str, start, end)
-char **str;
-int start, end;
+m_capture(char **str, int start, int end)
 {
-    register int i, e;
-    register char *s;
+    int i, e;
+    char *s;
 
     e = token[end].start_index + token[end].length;
     *str = gp_realloc(*str, (e - token[start].start_index + 1), "string");
     s = *str;
-    for (i = token[start].start_index; i < e && input_line[i] != NUL; i++)
-	*s++ = input_line[i];
+    for (i = token[start].start_index; i < e && gp_input_line[i] != NUL; i++)
+	*s++ = gp_input_line[i];
     *s = NUL;
 }
 
@@ -316,25 +320,55 @@ int start, end;
  * quotes from either end of the string.
  */
 void
-m_quote_capture(str, start, end)
-char **str;
-int start, end;
+m_quote_capture(char **str, int start, int end)
 {
-    register int i, e;
-    register char *s;
+    int i, e;
+    char *s;
 
     e = token[end].start_index + token[end].length - 1;
     *str = gp_realloc(*str, (e - token[start].start_index + 1), "string");
     s = *str;
-    for (i = token[start].start_index + 1; i < e && input_line[i] != NUL; i++)
-	*s++ = input_line[i];
+    for (i = token[start].start_index + 1; i < e && gp_input_line[i] != NUL; i++)
+	*s++ = gp_input_line[i];
     *s = NUL;
 
-    if (input_line[token[start].start_index] == '"')
+    if (gp_input_line[token[start].start_index] == '"')
 	parse_esc(*str);
+    else
+        parse_sq(*str);
 
 }
 
+/*
+ * Wrapper for isstring + m_quote_capture that can be used with
+ * or without GP_STRING_VARS enabled.
+ * EAM Aug 2004
+ */
+char *
+try_to_get_string()
+{
+    char *newstring = NULL;
+
+#ifdef GP_STRING_VARS
+    struct value a;
+    int save_token = c_token;
+    if (END_OF_COMMAND)
+	return NULL;
+    const_string_express(&a);
+    if (a.type == STRING)
+	newstring = a.v.string_val;
+    else
+	c_token = save_token;
+#else
+    if (!END_OF_COMMAND && isstring(c_token)) {
+	m_quote_capture(&newstring, c_token, c_token);
+	c_token++;
+    }
+#endif
+
+    return newstring;
+}
+
 
 /* Our own version of strdup()
  * Make copy of string into gp_alloc'd memory
@@ -343,8 +377,7 @@ int start, end;
  * valid parameters!
  */
 char *
-gp_strdup(s)
-const char *s;
+gp_strdup(const char *s)
 {
     char *d;
 
@@ -358,6 +391,18 @@ const char *s;
     return d;
 }
 
+/*
+ * Allocate a new string and initialize it by concatenating two
+ * existing strings.
+ */
+char *
+gp_stradd(const char *a, const char *b)
+{
+    char *new = gp_alloc(strlen(a)+strlen(b)+1,"gp_stradd");
+    strcpy(new,a);
+    strcat(new,b);
+    return new;
+}
 
 /* HBB 20020405: moved these functions here from axis.c, where they no
  * longer truly belong. */
@@ -365,12 +410,13 @@ const char *s;
 /* HBB 20010121: added code that attempts to fix rounding-induced
  * off-by-one errors in 10^%T and similar output formats */
 static void
-mant_exp(log10_base, x, scientific, m, p, format)
-    double log10_base, x;
-    TBOOLEAN scientific;	/* round to power of 3 */
-    double *m;			/* results */
-    int *p;			
-    const char *format;		/* format string for fixup */
+mant_exp(
+    double log10_base,
+    double x,
+    TBOOLEAN scientific,	/* round to power of 3 */
+    double *m,			/* results */
+    int *p,
+    const char *format)		/* format string for fixup */
 {
     int sign = 1;
     double l10;
@@ -395,55 +441,69 @@ mant_exp(log10_base, x, scientific, m, p, format)
 
     l10 = log10(x) / log10_base;
     power = floor(l10);
-    mantissa = pow(10.0, (l10 - power) * log10_base);
+    mantissa = pow(10.0, log10_base * (l10 - power));
 
     /* round power to an integer multiple of 3, to get what's
      * sometimes called 'scientific' or 'engineering' notation. Also
      * useful for handling metric unit prefixes like 'kilo' or 'micro'
      * */
-    /* HBB 20010121: avoid recalculation of mantissa via pow() */
     if (scientific) {
-	int temp_power  = 3 * floor(power / 3.0);
-	switch (power - temp_power) {
+	/* Scientific mode makes no sense whatsoever if the base of
+	 * the logarithmic axis is anything but 10.0 */
+	assert(log10_base == 1.0);
+
+	/* HBB FIXED 20040701: negative modulo positive may yield
+	 * negative result.  But we always want an effectively
+	 * positive modulus --> adjust input by one step */
+	switch (power % 3) {
+	case -1:
+	    power -= 3;
 	case 2:
-	    mantissa *= 100; break;
+	    mantissa *= 100;
+	    break;
+	case -2:
+	    power -= 3;
 	case 1:
-	    mantissa *= 10; break;
+	    mantissa *= 10;
+	    break;
 	case 0:
 	    break;
 	default:
 	    int_error (NO_CARET, "Internal error in scientific number formatting");
 	}
-	power = temp_power;
+	power -= (power % 3);
     }
 
     /* HBB 20010121: new code for decimal mantissa fixups.  Looks at
      * format string to see how many decimals will be put there.  Iff
-     * the number is so close to an exact power of ten that it will be
+     * the number is so close to an exact power of 10 that it will be
      * rounded up to 10.0e??? by an sprintf() with that many digits of
      * precision, increase the power by 1 to get a mantissa in the
      * region of 1.0.  If this handling is not wanted, pass NULL as
      * the format string */
+    /* HBB 20040521: extended to also work for bases other than 10.0 */
     if (format) {
-	double upper_border = scientific ? 1000 : 10;
+	double actual_base = (scientific ? 1000 : pow(10.0, log10_base));
 	int precision = 0;
+	double tolerance;
 
 	format = strchr (format, '.');
 	if (format != NULL)
-	    /* a decimal point was found in the format, so use that 
+	    /* a decimal point was found in the format, so use that
 	     * precision. */
 	    precision = strtol(format + 1, NULL, 10);
-	
-	/* See if mantissa would be right on the border.  All numbers
-	 * greater than that will be rounded up to 10, by sprintf(), which
-	 * we want to avoid. */
-	if (mantissa > (upper_border - pow(10.0, -precision) / 2)
-	    ) {
-	    mantissa /= upper_border;
+
+	/* See if mantissa would be right on the border.  The
+	 * condition to watch out for is that the mantissa is within
+	 * one printing precision of the next power of the logarithm
+	 * base.  So add the 0.5*10^-precision to the mantissa, and
+	 * see if it's now larger than the base of the scale */
+	tolerance = pow(10.0, -precision) / 2;
+	if (mantissa + tolerance >= actual_base) {
+	    mantissa /= actual_base;
 	    power += (scientific ? 3 : 1);
 	}
     }
-
     if (m)
 	*m = sign * mantissa;
     if (p)
@@ -473,10 +533,12 @@ mant_exp(log10_base, x, scientific, m, p, format)
  * only, with logscaled axes, in combination with the occasional
  * round-off error. */
 void
-gprintf(dest, count, format, log10_base, x)
-    char *dest, *format;
-    size_t count;
-    double log10_base, x;
+gprintf(
+    char *dest,
+    size_t count,
+    char *format,
+    double log10_base,
+    double x)
 {
     char temp[MAX_LINE_LEN + 1];
     char *t;
@@ -505,7 +567,8 @@ gprintf(dest, count, format, log10_base, x)
 	*t++ = '%';
 	/* dont put isdigit first since sideeffect in macro is bad */
 	while (*++format == '.' || isdigit((unsigned char) *format)
-	       || *format == '-' || *format == '+' || *format == ' ')
+	       || *format == '-' || *format == '+' || *format == ' '
+	       || *format == '\'')
 	    *t++ = *format;
 	/*}}} */
 
@@ -596,7 +659,7 @@ gprintf(dest, count, format, log10_base, x)
 		t[1] = 0;
 		if (seen_mantissa)
 		    power = stored_power;
-		else 
+		else
 		    mant_exp(1.0, x, FALSE, NULL, &power, "%.0f");
 		sprintf(dest, temp, power);
 		break;
@@ -611,7 +674,7 @@ gprintf(dest, count, format, log10_base, x)
 		t[1] = 0;
 		if (seen_mantissa)
 		    power = stored_power;
-		else 
+		else
 		    mant_exp(1.0, x, TRUE, NULL, &power, "%.0f");
 		sprintf(dest, temp, power);
 		break;
@@ -626,7 +689,7 @@ gprintf(dest, count, format, log10_base, x)
 		t[1] = 0;
 		if (seen_mantissa)
 		    power = stored_power;
-		else 
+		else
 		    mant_exp(1.0, x, TRUE, NULL, &power, "%.0f");
 
 		if (power >= -18 && power <= 18) {
@@ -670,9 +733,17 @@ gprintf(dest, count, format, log10_base, x)
 	if (decimalsign != NULL) {
 	    char *dotpos1 = dest, *dotpos2;
 	    size_t newlength = strlen(decimalsign);
+	    int dot;
 
-	    /* replace every `.' by the contents of decimalsign */
-	    while ((dotpos2 = strchr(dotpos1,'.')) != NULL) {
+	    /* dot is the default decimalsign we will be replacing */
+#ifdef HAVE_LOCALE_H
+	    dot = *(localeconv()->decimal_point);
+#else
+	    dot = '.';
+#endif
+
+	    /* replace every dot by the contents of decimalsign */
+	    while ((dotpos2 = strchr(dotpos1,dot)) != NULL) {
 		size_t taillength = strlen(dotpos2);
 
 		dotpos1 = dotpos2 + newlength;
@@ -704,26 +775,39 @@ gprintf(dest, count, format, log10_base, x)
 /* some macros for the error and warning functions below
  * may turn this into a utility function later
  */
-#define PRINT_SPACES_UNDER_PROMPT \
-{ register size_t i; \
-  for (i = 0; i < sizeof(PROMPT) - 1; i++) \
-   (void) fputc(' ', stderr); \
-}
+#define PRINT_MESSAGE_TO_STDERR				\
+do {							\
+    fprintf(stderr, "\n%s%s\n",				\
+	    current_prompt ? current_prompt : "",	\
+	    gp_input_line);				\
+} while (0)
+    
+#define PRINT_SPACES_UNDER_PROMPT		\
+do {						\
+    const char *p;				\
+						\
+    if (!current_prompt)			\
+	break;					\
+    for (p = current_prompt; *p != '\0'; p++)	\
+	(void) fputc(' ', stderr);		\
+} while (0)
 
-#define PRINT_SPACES_UPTO_TOKEN \
-{ register int i; \
-   for (i = 0; i < token[t_num].start_index; i++) \
-    (void) fputc((input_line[i] == '\t') ? '\t' : ' ', stderr); \
-}
+#define PRINT_SPACES_UPTO_TOKEN						\
+do {									\
+    int i;								\
+									\
+    for (i = 0; i < token[t_num].start_index; i++)			\
+	(void) fputc((gp_input_line[i] == '\t') ? '\t' : ' ', stderr);	\
+} while(0)
 
 #define PRINT_CARET fputs("^\n",stderr);
 
-#define PRINT_FILE_AND_LINE \
- if (!interactive) { \
-        if (infile_name != NULL) \
-            fprintf(stderr, "\"%s\", line %d: ", infile_name, inline_num); \
-        else fprintf(stderr, "line %d: ", inline_num); \
- }
+#define PRINT_FILE_AND_LINE						\
+if (!interactive) {							\
+    if (infile_name != NULL)						\
+	fprintf(stderr, "\"%s\", line %d: ", infile_name, inline_num);	\
+    else fprintf(stderr, "line %d: ", inline_num);			\
+}
 
 /* TRUE if command just typed; becomes FALSE whenever we
  * send some other output to screen.  If FALSE, the command line
@@ -736,10 +820,7 @@ void
 os_error(int t_num, const char *str,...)
 #else
 void
-os_error(t_num, str, va_alist)
-int t_num;
-const char *str;
-va_dcl
+os_error(int t_num, const char *str, va_dcl)
 #endif
 {
 #ifdef VA_START
@@ -755,8 +836,8 @@ va_dcl
 	df_showdata();
     } else if (t_num != NO_CARET) {	/* put caret under error */
 	if (!screen_ok)
-	    fprintf(stderr, "\n%s%s\n", PROMPT, input_line);
-	
+	    PRINT_MESSAGE_TO_STDERR;
+
 	PRINT_SPACES_UNDER_PROMPT;
 	PRINT_SPACES_UPTO_TOKEN;
 	PRINT_CARET;
@@ -797,10 +878,7 @@ void
 int_error(int t_num, const char *str,...)
 #else
 void
-int_error(t_num, str, va_alist)
-int t_num;
-const char str[];
-va_dcl
+int_error(int t_num, const char str[], va_dcl)
 #endif
 {
 #ifdef VA_START
@@ -813,7 +891,7 @@ va_dcl
         df_showdata();
     } else if (t_num != NO_CARET) { /* put caret under error */
 	if (!screen_ok)
-	    fprintf(stderr, "\n%s%s\n", PROMPT, input_line);
+	    PRINT_MESSAGE_TO_STDERR;
 
 	PRINT_SPACES_UNDER_PROMPT;
 	PRINT_SPACES_UPTO_TOKEN;
@@ -835,6 +913,12 @@ va_dcl
 #endif
     fputs("\n\n", stderr);
 
+    /* We are bailing out of nested context without ever reaching */
+    /* the normal cleanup code. Reset any flags before bailing.   */
+    df_reset_after_error();
+
+    update_gpval_variables(2);
+
     bail_to_command_line();
 }
 
@@ -844,10 +928,7 @@ void
 int_warn(int t_num, const char *str,...)
 #else
 void
-int_warn(t_num, str, va_alist)
-int t_num;
-const char str[];
-va_dcl
+int_warn(int t_num, const char str[], va_dcl)
 #endif
 {
 #ifdef VA_START
@@ -860,7 +941,7 @@ va_dcl
         df_showdata();
     } else if (t_num != NO_CARET) { /* put caret under error */
 	if (!screen_ok)
-	    fprintf(stderr, "\n%s%s\n", PROMPT, input_line);
+	    PRINT_MESSAGE_TO_STDERR;
 
 	PRINT_SPACES_UNDER_PROMPT;
 	PRINT_SPACES_UPTO_TOKEN;
@@ -878,11 +959,11 @@ va_dcl
     _doprnt(str, args, stderr);
 # endif
     va_end(args);
-#else
+#else  /* VA_START */
     fprintf(stderr, str, a1, a2, a3, a4, a5, a6, a7, a8);
-#endif
+#endif /* VA_START */
     putc('\n', stderr);
-}				/* int_warn */
+}
 
 /*{{{  graph_error() */
 /* handle errors during graph-plot in a consistent way */
@@ -892,9 +973,7 @@ void
 graph_error(const char *fmt, ...)
 #else
 void
-graph_error(fmt, va_alist)
-    const char *fmt;
-    va_dcl
+graph_error(const char *fmt, va_dcl)
 #endif
 {
 #ifdef VA_START
@@ -906,7 +985,7 @@ graph_error(fmt, va_alist)
 
 #ifdef VA_START
     VA_START(args, fmt);
-#if 0 
+#if 0
     /* HBB 20001120: this seems not to work at all. Probably because a
      * va_list argument, is, after all, something else than a varargs
      * list (i.e. a '...') */
@@ -940,14 +1019,14 @@ graph_error(fmt, va_alist)
 /* Lower-case the given string (DFK) */
 /* Done in place. */
 void
-lower_case(s)
-char *s;
+lower_case(char *s)
 {
-    register char *p = s;
+    char *p = s;
 
-    while (*p++) {
+    while (*p) {
 	if (isupper((unsigned char)*p))
 	    *p = tolower((unsigned char)*p);
+	p++;
     }
 }
 
@@ -955,11 +1034,10 @@ char *s;
 /* That is, reduce all multiple white-space chars to single spaces */
 /* Done in place. */
 void
-squash_spaces(s)
-char *s;
+squash_spaces(char *s)
 {
-    register char *r = s;	/* reading point */
-    register char *w = s;	/* writing point */
+    char *r = s;	/* reading point */
+    char *w = s;	/* writing point */
     TBOOLEAN space = FALSE;	/* TRUE if we've already copied a space */
 
     for (w = r = s; *r != NUL; r++) {
@@ -979,9 +1057,28 @@ char *s;
 }
 
 
-static void
-parse_esc(instr)
-char *instr;
+/* postprocess single quoted strings: replace "''" by "'"
+*/
+void
+parse_sq(char *instr)
+{
+    char *s = instr, *t = instr;
+
+    /* the string will always get shorter, so we can do the
+     * conversion in situ
+     */
+
+    while (*s != NUL) {
+        if (*s == '\'' && *(s+1) == '\'')
+            s++;
+        *t++ = *s++;
+    }
+    *t = NUL;
+}
+
+
+void
+parse_esc(char *instr)
 {
     char *s = instr, *t = instr;
 
@@ -1009,7 +1106,8 @@ char *instr;
 		s++;
 	    } else if (*s >= '0' && *s <= '7') {
 		int i, n;
-		if (sscanf(s, "%o%n", &i, &n) > 0) {
+		char *octal = (*s == '0' ? "%4o%n" : "%3o%n");
+		if (sscanf(s, octal, &i, &n) > 0) {
 		    *t++ = i;
 		    s += n;
 		} else {
@@ -1018,6 +1116,9 @@ char *instr;
 		    *t++ = *s++;
 		}
 	    }
+	} else if (df_separator && *s == '\"' && *(s+1) == '\"') {
+	/* EAM Mar 2003 - For parsing CSV strings with quoted quotes */
+	    *t++ = *s++; s++;
 	} else {
 	    *t++ = *s++;
 	}
@@ -1026,17 +1127,16 @@ char *instr;
 }
 
 
-/* FIXME HH 20020915: This function does nothing if dirent.h and windows.h 
+/* FIXME HH 20020915: This function does nothing if dirent.h and windows.h
  * not available. */
-TBOOLEAN 
-existdir (name)
-     const char *name;
+TBOOLEAN
+existdir (const char *name)
 {
 #ifdef HAVE_DIRENT_H
     DIR *dp;
     if (! (dp = opendir(name) ) )
 	return FALSE;
-    
+
     closedir(dp);
     return TRUE;
 #elif defined(_Windows)
@@ -1049,6 +1149,8 @@ existdir (name)
 	    return TRUE;
     }
     return FALSE;
+#elif defined(VMS)
+    return FALSE;
 #else
     int_warn(NO_CARET,
 	     "Test on directory existence not supported\n\t('%s!')",
@@ -1057,3 +1159,41 @@ existdir (name)
 #endif
 }
 
+char *
+getusername ()
+{
+    char *username = NULL;
+    char *fullname = NULL;
+
+    username=getenv("USER");
+    if (!username)
+	username=getenv("USERNAME");
+
+#ifdef HAVE_PWD_H
+    if (username) {
+	struct passwd *pwentry = NULL;
+	pwentry=getpwnam(username);
+	if (pwentry && strlen(pwentry->pw_gecos)) {
+	    fullname = gp_alloc(strlen(pwentry->pw_gecos)+1,"getusername");
+	    strcpy(fullname, pwentry->pw_gecos);
+	} else {
+	    fullname = gp_alloc(strlen(username)+1,"getusername");
+	    strcpy(fullname, username);
+	}
+    }
+#elif defined(_Windows)
+    if (username) {
+	DWORD bufCharCount = INFO_BUFFER_SIZE;
+	fullname = gp_alloc(INFO_BUFFER_SIZE + 1,"getusername");
+	if (!GetUserName(fullname,&bufCharCount)) {
+	    free(fullname);
+	    fullname = NULL;
+	}
+    }
+#else
+    fullname = gp_alloc(strlen(username)+1,"getusername");
+    strcpy(fullname, username);
+#endif /* HAVE_PWD_H */
+
+    return fullname;
+}
