@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: version.c,v 1.69.2.8 2007/03/03 06:22:13 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: version.c,v 1.69.2.12 2007/08/26 06:15:51 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - version.c */
@@ -40,15 +40,14 @@ static char *RCSid() { return RCSid("$Id: version.c,v 1.69.2.8 2007/03/03 06:22:
 
 
 const char gnuplot_version[] = "4.2";
-const char gnuplot_patchlevel[] = "0";
-const char gnuplot_date[] = "March 2007";
+const char gnuplot_patchlevel[] = "2 ";
+const char gnuplot_date[] = "31 Aug 2007";
 const char gnuplot_copyright[] = "Copyright (C) 1986 - 1993, 1998, 2004, 2007";
 
 const char faq_location[] = FAQ_LOCATION;
 
 char *compile_options = (void *)0;	/* Will be loaded at runtime */
 
-/* Disable this during development series; enable for release */
 #define RELEASE_VERSION 1
 
 /* mustn't forget to activate this before the release ... */
@@ -57,7 +56,8 @@ char *compile_options = (void *)0;	/* Will be loaded at runtime */
 #  define HELPMAIL "gnuplot-info@lists.sourceforge.net";
 # endif
 # ifndef CONTACT
-#  define CONTACT "gnuplot-bugs@lists.sourceforge.ent";
+/* #  define CONTACT "gnuplot-bugs@lists.sourceforge.net"; */
+#  define CONTACT "http://sourceforge.net/projects/gnuplot";
 # endif
 
 const char bug_email[] = CONTACT;
